@@ -26,11 +26,7 @@ def set_debug_flag(flag):
 	Determine whether Core module runs in debugging mode or not.
 	When in debugging mode (flag = true), extra output is produced when certain conditions in the code are not met
 	"""
-	if not isinstance(flag, (bool)):
-		raise Exception("flag argument must be of class bool")
-	pma._pma_debug = flag
-	if flag == True:
-		print("Debug flag enabled. You will receive extra feedback and messages from pma_python (like this one)")
+	pma._pma_set_debug_flag(flag)
 
 def _pma_session_id(sessionID = None):
 	if (sessionID is None):
@@ -107,6 +103,15 @@ def _pma_api_url(sessionID = None, xml = True):
 	else:
 		return pma._pma_join(url, "api/json/")
 
+def _pma_query_url(sessionID = None):
+	# let's get the base URL first for the specified session
+	url = _pma_url(sessionID)
+	if url is None:
+		# sort of a hopeless situation; there is no URL to refer to
+		return None
+	# remember, _pma_url is guaranteed to return a URL that ends with "/"
+	return pma._pma_join(url, "query/json/")
+
 def _pma_XmlToStringArray(root, limit = 0):
 	els = root.getElementsByTagName("string")
 	l = []
@@ -188,7 +193,9 @@ def connect(pmacoreURL = _pma_pmacoreliteURL, pmacoreUsername = "", pmacorePassw
 		sessionID = None
 	else:
 		sessionID = loginresult["SessionId"]
-				
+		if (pma._pma_debug == True):
+			print("sessionID =", sessionID)
+			
 		global _pma_sessions						# so afterwards we can look up what username actually belongs to a sessions	
 		global _pma_usernames						# so afterwards we can determine the PMA.core URL to connect to for a given SessionID
 		global _pma_slideinfos						# a caching mechanism for slide information; obsolete and should be improved through pma._pma_http_get()
@@ -197,9 +204,11 @@ def connect(pmacoreURL = _pma_pmacoreliteURL, pmacoreUsername = "", pmacorePassw
 		_pma_usernames[sessionID] = pmacoreUsername		
 		_pma_sessions[sessionID] = pmacoreURL			
 		if not (sessionID in _pma_slideinfos):
-			_pma_slideinfos[sessionID] = dict()
+			_pma_slideinfos[sessionID] = {}
 		_pma_amount_of_data_downloaded[sessionID] = len(loginresult)
 	
+	if (pma._pma_debug == True):
+		print("Final sessionID to be returned =", sessionID)
 	return sessionID
 
 def disconnect(sessionID = None):
@@ -882,3 +891,32 @@ def get_files_for_slide(slideRef, sessionID = None):
 			retval[file["Path"]] = { "Size": file["Size"], "LastModified": file["LastModified"] }
 			
 	return retval
+
+def search_slides(startDir, pattern, sessionID = None):
+	sessionID = _pma_session_id(sessionID)
+	
+	sessionID = _pma_session_id(sessionID)
+	if (sessionID == _pma_pmacoreliteSessionID):
+		if is_lite():
+			raise ValueError ("PMA.core.lite found running, but doesn't support searching.")
+		else:
+			raise ValueError ("PMA.core.lite not found, and besides; it doesn't support searching.")
+
+	if (startDir.startswith("/")):
+		startDir = slideRef[1:]		
+
+	url = _pma_query_url(sessionID) + "Filename?sessionID=" + pma._pma_q(sessionID) + "&path=" + pma._pma_q(startDir) + "&pattern=" + pma._pma_q(pattern)
+	if pma._pma_debug == True:
+		print("url =", url)
+	
+	r = requests.get(url)
+	json = r.json()
+	global _pma_amount_of_data_downloaded 
+	_pma_amount_of_data_downloaded[sessionID] += len(json)
+	if ("Code" in json):
+		raise Exception("enumerate_files_for_slide on  " + slideRef + " resulted in: " + json["Message"] + " (keep in mind that slideRef is case sensitive!)")
+	elif ("d" in json):
+		files  = json["d"]
+	else:
+		files = json
+	return files
